@@ -28,7 +28,7 @@ class BaseFormatter(Generic[T]):
         display_relative_path (bool): whether to show path as relative or absolute
     """
 
-    def __init__(self, base_dir: Union[str, Path], display_relative_path: bool) -> None:
+    def __init__(self, base_dir: str | Path, display_relative_path: bool) -> None:
         """Initialize a BaseFormatter instance."""
         if isinstance(base_dir, str):
             base_dir = Path(base_dir)
@@ -37,7 +37,7 @@ class BaseFormatter(Generic[T]):
 
         self._base_dir = base_dir if display_relative_path else None
 
-    def _format_path(self, path: Union[str, Path]) -> Union[str, Path]:
+    def _format_path(self, path: str | Path) -> str | Path:
         if not self._base_dir or not path:
             return path
         # Use os.path.relpath 'cause Path.relative_to() misbehaves
@@ -58,15 +58,15 @@ class Formatter(BaseFormatter):  # type: ignore
 
     def format(self, match: MatchError) -> str:
         _id = getattr(match.rule, "id", "000")
-        result = f"[error_code]{_id}[/][dim]:[/] [error_title]{self.escape(match.message)}[/]"
-        if match.tag:
-            result += f" [dim][error_code]({self.escape(match.tag)})[/][/]"
+        result = f"[{match.level}][bold][link={match.rule.url}]{self.escape(match.tag)}[/link][/][/][dim]:[/] [{match.level}]{self.escape(match.message)}[/]"
+        if match.level != "error":
+            result += f" [dim][{match.level}]({match.level})[/][/]"
         result += (
             "\n"
             f"[filename]{self._format_path(match.filename or '')}[/]:{match.position}"
         )
         if match.details:
-            result += f" [dim]{match.details}[/]"
+            result += f" [dim]{self.escape(str(match.details))}[/]"
         result += "\n"
         return result
 
@@ -76,7 +76,7 @@ class QuietFormatter(BaseFormatter[Any]):
 
     def format(self, match: MatchError) -> str:
         return (
-            f"[error_code]{match.rule.id}[/] "
+            f"[{match.level}]{match.rule.id}[/] "
             f"[filename]{self._format_path(match.filename or '')}[/]:{match.position}"
         )
 
@@ -86,20 +86,20 @@ class ParseableFormatter(BaseFormatter[Any]):
 
     def format(self, match: MatchError) -> str:
         result = (
-            f"[filename]{self._format_path(match.filename or '')}[/]:{match.position}: "
-            f"[error_code]{match.rule.id}[/]"
+            f"[filename]{self._format_path(match.filename or '')}[/][dim]:{match.position}:[/] "
+            f"[{match.level}][bold]{self.escape(match.tag)}[/bold]"
+            f": {match.message}"
+            if not options.quiet
+            else "[/]"
         )
+        if match.level != "error":
+            result += f" [dim][{match.level}]({match.level})[/][/]"
 
-        if not options.quiet:
-            result += f": [dim]{match.message}[/]"
-
-        if match.tag:
-            result += f" [dim][error_code]({self.escape(match.tag)})[/][/]"
         return result
 
 
 class AnnotationsFormatter(BaseFormatter):  # type: ignore
-    # https://docs.github.com/en/actions/reference/workflow-commands-for-github-actions#setting-a-warning-message
+    # https://docs.github.com/en/actions/using-workflows/workflow-commands-for-github-actions#setting-a-warning-message
     """Formatter for emitting violations as GitHub Workflow Commands.
 
     These commands trigger the GHA Workflow runners platform to post violations
@@ -115,10 +115,8 @@ class AnnotationsFormatter(BaseFormatter):  # type: ignore
 
     def format(self, match: MatchError) -> str:
         """Prepare a match instance for reporting as a GitHub Actions annotation."""
-        level = self._severity_to_level(match.rule.severity)
         file_path = self._format_path(match.filename or "")
         line_num = match.linenumber
-        rule_id = match.rule.id
         severity = match.rule.severity
         violation_details = self.escape(match.message)
         if match.column:
@@ -126,19 +124,10 @@ class AnnotationsFormatter(BaseFormatter):  # type: ignore
         else:
             col = ""
         return (
-            f"::{level} file={file_path},line={line_num}{col},severity={severity}"
-            f"::{rule_id} {violation_details}"
+            f"::{match.level} file={file_path},line={line_num}{col},severity={severity},title={match.tag}"
+            f"::{violation_details}"
         )
 
-    @staticmethod
-    def _severity_to_level(severity: str) -> str:
-        if severity in ["VERY_LOW", "LOW"]:
-            return "warning"
-        if severity in ["INFO"]:
-            return "debug"
-        # ['MEDIUM', 'HIGH', 'VERY_HIGH'] or anything else
-        return "error"
-
 
 class CodeclimateJSONFormatter(BaseFormatter[Any]):
     """Formatter for emitting violations in Codeclimate JSON report format.
@@ -148,7 +137,7 @@ class CodeclimateJSONFormatter(BaseFormatter[Any]):
     https://github.com/codeclimate/platform/blob/master/spec/analyzers/SPEC.md#user-content-data-types
     """
 
-    def format_result(self, matches: List[MatchError]) -> str:
+    def format_result(self, matches: list[MatchError]) -> str:
         """Format a list of match errors as a JSON string."""
         if not isinstance(matches, list):
             raise RuntimeError(
@@ -157,11 +146,11 @@ class CodeclimateJSONFormatter(BaseFormatter[Any]):
 
         result = []
         for match in matches:
-            issue: Dict[str, Any] = {}
+            issue: dict[str, Any] = {}
             issue["type"] = "issue"
             issue["check_name"] = match.tag or match.rule.id  # rule-id[subrule-id]
             issue["categories"] = match.rule.tags
-            issue["severity"] = self._severity_to_level(match.rule.severity)
+            issue["severity"] = self._severity_to_level(match)
             issue["description"] = self.escape(str(match.message))
             issue["fingerprint"] = hashlib.sha256(
                 repr(match).encode("utf-8")
@@ -184,7 +173,11 @@ class CodeclimateJSONFormatter(BaseFormatter[Any]):
         return json.dumps(result)
 
     @staticmethod
-    def _severity_to_level(severity: str) -> str:
+    def _severity_to_level(match: MatchError) -> str:
+        if match.level != "error":
+            return "info"
+        severity = match.rule.severity
+
         if severity in ["LOW"]:
             return "minor"
         if severity in ["MEDIUM"]:
@@ -205,15 +198,14 @@ class SarifFormatter(BaseFormatter[Any]):
     """
 
     BASE_URI_ID = "SRCROOT"
-    TOOL_NAME = "Ansible-lint"
+    TOOL_NAME = "ansible-lint"
     TOOL_URL = "https://github.com/ansible/ansible-lint"
     SARIF_SCHEMA_VERSION = "2.1.0"
-    RULE_DOC_URL = "https://ansible-lint.readthedocs.io/en/latest/default_rules/"
     SARIF_SCHEMA = (
         "https://schemastore.azurewebsites.net/schemas/json/sarif-2.1.0-rtm.5.json"
     )
 
-    def format_result(self, matches: List[MatchError]) -> str:
+    def format_result(self, matches: list[MatchError]) -> str:
         """Format a list of match errors as a JSON string."""
         if not isinstance(matches, list):
             raise RuntimeError(
@@ -255,8 +247,8 @@ class SarifFormatter(BaseFormatter[Any]):
         )
 
     def _extract_results(
-        self, matches: List[MatchError]
-    ) -> Tuple[List[Any], List[Any]]:
+        self, matches: list[MatchError]
+    ) -> tuple[list[Any], list[Any]]:
         rules = {}
         results = []
         for match in matches:
@@ -265,28 +257,28 @@ class SarifFormatter(BaseFormatter[Any]):
             results.append(self._to_sarif_result(match))
         return list(rules.values()), results
 
-    def _to_sarif_rule(self, match: MatchError) -> Dict[str, Any]:
-        rule: Dict[str, Any] = {
+    def _to_sarif_rule(self, match: MatchError) -> dict[str, Any]:
+        rule: dict[str, Any] = {
             "id": match.rule.id,
-            "name": match.rule.id,
+            "name": match.tag,
             "shortDescription": {
                 "text": self.escape(str(match.message)),
             },
             "defaultConfiguration": {
-                "level": self._to_sarif_level(match.rule.severity),
+                "level": self._to_sarif_level(match),
             },
             "help": {
                 "text": str(match.rule.description),
             },
-            "helpUri": self.RULE_DOC_URL + "#" + match.rule.id,
+            "helpUri": match.rule.url,
             "properties": {"tags": match.rule.tags},
         }
         if match.rule.link:
             rule["helpUri"] = match.rule.link
         return rule
 
-    def _to_sarif_result(self, match: MatchError) -> Dict[str, Any]:
-        result: Dict[str, Any] = {
+    def _to_sarif_result(self, match: MatchError) -> dict[str, Any]:
+        result: dict[str, Any] = {
             "ruleId": match.rule.id,
             "message": {
                 "text": match.details,
@@ -312,10 +304,6 @@ class SarifFormatter(BaseFormatter[Any]):
         return result
 
     @staticmethod
-    def _to_sarif_level(severity: str) -> str:
-        if severity in ["VERY_HIGH", "HIGH", "MEDIUM"]:
-            return "error"
-        if severity in ["LOW"]:
-            return "warning"
-        # VERY_LOW, INFO or anything else
-        return "note"
+    def _to_sarif_level(match: MatchError) -> str:
+        # sarif accepts only 4 levels: error, warning, note, none
+        return match.level
