@@ -55,7 +55,16 @@ from ansiblelint._internal.rules import (
 )
 from ansiblelint.app import get_app
 from ansiblelint.config import options
-from ansiblelint.constants import NESTED_TASK_KEYS, PLAYBOOK_TASK_KEYWORDS, FileType
+from ansiblelint.constants import (
+    FILENAME_KEY,
+    INCLUSION_ACTION_NAMES,
+    LINE_NUMBER_KEY,
+    NESTED_TASK_KEYS,
+    PLAYBOOK_TASK_KEYWORDS,
+    ROLE_IMPORT_ACTION_NAMES,
+    SKIPPED_RULES_KEY,
+    FileType,
+)
 from ansiblelint.errors import MatchError
 from ansiblelint.file_utils import Lintable, discover_lintables
 from ansiblelint.skip_utils import is_nested_task
@@ -107,14 +116,31 @@ def ansible_templar(basedir: str, templatevars: Any) -> Templar:
 def ansible_template(
     basedir: str, varname: Any, templatevars: Any, **kwargs: Any
 ) -> Any:
-    """Render a templated string."""
+    """Render a templated string by mocking missing filters."""
     templar = ansible_templar(basedir=basedir, templatevars=templatevars)
-    return templar.template(varname, **kwargs)
+    # pylint: disable=unused-variable
+    for i in range(3):
+        try:
+            return templar.template(varname, **kwargs)
+        except AnsibleError as exc:
+            if (
+                exc.message.startswith("template error while templating string:")
+                and "'" in exc.message
+            ):
+                missing_filter = exc.message.split("'")[1]
+                if missing_filter == "end of print statement":
+                    raise
+                # Mock the filter to avoid and error from Ansible templating
+                # pylint: disable=protected-access
+                templar.environment.filters._delegatee[missing_filter] = lambda x: x
+                # Record the mocked filter so we can warn the user
+                if missing_filter not in options.mock_filters:
+                    _logger.debug("Mocking missing filter %s", missing_filter)
+                    options.mock_filters.append(missing_filter)
+                continue
+            raise
 
 
-LINE_NUMBER_KEY = "__line__"
-FILENAME_KEY = "__file__"
-
 BLOCK_NAME_TO_ACTION_TYPE_MAP = {
     "tasks": "task",
     "handlers": "handler",
@@ -215,6 +241,7 @@ def template(
     basedir: str,
     value: Any,
     variables: Any,
+    fail_on_error: bool = False,
     fail_on_undefined: bool = False,
     **kwargs: str,
 ) -> Any:
@@ -230,7 +257,8 @@ def template(
         # I guess the filter doesn't like empty vars...
     except (AnsibleError, ValueError, RepresenterError):
         # templating failed, so just keep value as is.
-        pass
+        if fail_on_error:
+            raise
     return value
 
 
@@ -275,11 +303,7 @@ def _include_children(
     basedir: str, k: str, v: Any, parent_type: FileType
 ) -> list[Lintable]:
     # handle special case include_tasks: name=filename.yml
-    if (
-        k in ["include_tasks", "ansible.builtin.include_tasks"]
-        and isinstance(v, dict)
-        and "file" in v
-    ):
+    if k in INCLUSION_ACTION_NAMES and isinstance(v, dict) and "file" in v:
         v = v["file"]
 
     # we cannot really parse any jinja2 in includes, so we ignore them
@@ -330,13 +354,7 @@ def _taskshandlers_children(
             results.append(children)
             continue
 
-        including_commands = (
-            "import_role",
-            "ansible.builtin.import_role",
-            "include_role",
-            "ansible.builtin.include_role",
-        )
-        if any(x in task_handler for x in including_commands):
+        if any(x in task_handler for x in ROLE_IMPORT_ACTION_NAMES):
             # lgtm [py/unreachable-statement]
             task_handler = normalize_task_v2(task_handler)
             _validate_task_handler_action_for_role(task_handler["action"])
@@ -379,17 +397,7 @@ def _get_task_handler_children_for_tasks_or_playbooks(
     child_type = k if parent_type == "playbook" else parent_type
 
     # Include the FQCN task names as this happens before normalize
-    task_include_keys = (
-        "include",
-        "include_tasks",
-        "import_playbook",
-        "import_tasks",
-        "ansible.builtin.include",
-        "ansible.builtin.include_tasks",
-        "ansible.builtin.import_playbook",
-        "ansible.builtin.import_tasks",
-    )
-    for task_handler_key in task_include_keys:
+    for task_handler_key in INCLUSION_ACTION_NAMES:
 
         with contextlib.suppress(KeyError):
 
@@ -402,10 +410,15 @@ def _get_task_handler_children_for_tasks_or_playbooks(
                 file_name = file_name["file"]
 
             f = path_dwim(basedir, file_name)
+            while basedir not in ["", "/"]:
+                if os.path.exists(f):
+                    break
+                basedir = os.path.dirname(basedir)
+                f = path_dwim(basedir, file_name)
             return Lintable(f, kind=child_type)
 
     raise LookupError(
-        f'The node contains none of: {", ".join(task_include_keys)}',
+        f'The node contains none of: {", ".join(sorted(INCLUSION_ACTION_NAMES))}',
     )
 
 
@@ -511,7 +524,7 @@ def _sanitize_task(task: dict[str, Any]) -> dict[str, Any]:
     result = task.copy()
     # task is an AnsibleMapping which inherits from OrderedDict, so we need
     # to use `del` to remove unwanted keys.
-    for k in ["skipped_rules", FILENAME_KEY, LINE_NUMBER_KEY]:
+    for k in [SKIPPED_RULES_KEY, FILENAME_KEY, LINE_NUMBER_KEY]:
         if k in result:
             del result[k]
     return result
@@ -619,23 +632,23 @@ def task_to_str(task: dict[str, Any]) -> str:
     action = task.get("action")
     if isinstance(action, str) or not isinstance(action, dict):
         return str(action)
-    args = " ".join(
-        [
-            f"{k}={v}"
-            for (k, v) in action.items()
-            if k
-            not in [
-                "__ansible_module__",
-                "__ansible_module_original__",
-                "__ansible_arguments__",
-                LINE_NUMBER_KEY,
-                FILENAME_KEY,
-            ]
+    args = [
+        f"{k}={v}"
+        for (k, v) in action.items()
+        if k
+        not in [
+            "__ansible_module__",
+            "__ansible_module_original__",
+            "__ansible_arguments__",
+            LINE_NUMBER_KEY,
+            FILENAME_KEY,
         ]
-    )
+    ]
+
     for item in action.get("__ansible_arguments__", []):
-        args += f" {item}"
-    return f"{action['__ansible_module__']} {args}"
+        args.append(str(item))
+
+    return f"{action['__ansible_module__']} {' '.join(args)}"
 
 
 def extract_from_list(
@@ -683,21 +696,7 @@ def get_action_tasks(data: AnsibleBaseYAMLObject, file: Lintable) -> list[Any]:
     # Add sub-elements of block/rescue/always to tasks list
     tasks.extend(extract_from_list(tasks, NESTED_TASK_KEYS, recursive=True))
 
-    # Include the FQCN task names as this happens before normalize
-    return [
-        task
-        for task in tasks
-        if {
-            "include",
-            "include_tasks",
-            "import_playbook",
-            "import_tasks",
-            "ansible.builtin.include",
-            "ansible.builtin.include_tasks",
-            "ansible.builtin.import_playbook",
-            "ansible.builtin.import_tasks",
-        }.isdisjoint(task.keys())
-    ]
+    return tasks
 
 
 @lru_cache(maxsize=None)
